# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os

#inp
from inp import inp_get_token_value

#path
from cal_path import get_materials_path

from gl import glWidget
from gl_fallback import gl_fallback
#qt
from PyQt5.QtWidgets import QMainWindow, QTextEdit, QAction, QApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QWidget,QSizePolicy,QVBoxLayout,QPushButton,QDialog,QFileDialog,QToolBar,QMessageBox, QLineEdit,QLabel
from PyQt5.QtCore import QTimer
from PyQt5.QtCore import pyqtSignal

from icon_lib import QIcon_load

from help import help_window
from gl_cmp import gl_cmp

from util import str2bool
from fx_selector import fx_selector

from cal_path import get_sim_path
from global_objects import global_object_register
from global_objects import global_object_run

open_gl_working=False

def is_open_gl_working():
	global open_gl_working
	if open_gl_working==True:
		return "yes"
	else:
		return "no"

class display_widget(QWidget):

	colors=[]

	def add_fallback(self):
		global open_gl_working
		open_gl_working=False
		self.tb_rotate.setEnabled(False)
		self.xy.setEnabled(False)
		self.yz.setEnabled(False)
		self.xz.setEnabled(False)

		self.display=gl_fallback()
		self.hbox.addWidget(self.display)

	def update_ray_file(self):
		self.display.ray_file=os.path.join(get_sim_path(),"light_dump","light_ray_"+self.fx_box.get_text()+".dat")

	def __init__(self):
		QWidget.__init__(self)
		self.complex_display=False

		self.hbox=QVBoxLayout()
		self.gl_cmp=gl_cmp(os.path.join(get_sim_path(),"snapshots"))

		toolbar=QToolBar()
		toolbar.setIconSize(QSize(42, 42))

		#self.tb_layer_editor = QAction(QIcon_load("layers"), _("Layer\neditor"), self)
		#self.tb_layer_editor.triggered.connect(self.callback_layer_editor)
		#toolbar.addAction(self.tb_layer_editor)



		spacer = QWidget()
		spacer.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
		toolbar.addWidget(spacer)

		self.xy = QAction(QIcon_load("xy"), _("xy"), self)
		self.xy.triggered.connect(self.callback_xy)
		toolbar.addAction(self.xy)

		self.yz = QAction(QIcon_load("yz"), _("yz"), self)
		self.yz.triggered.connect(self.callback_yz)
		toolbar.addAction(self.yz)

		self.xz = QAction(QIcon_load("xz"), _("xz"), self)
		self.xz.triggered.connect(self.callback_xz)
		toolbar.addAction(self.xz)

		self.tb_rotate = QAction(QIcon_load("rotate.png"), _("Rotate"), self)
		self.tb_rotate.triggered.connect(self.tb_rotate_click)
		toolbar.addAction(self.tb_rotate)
		self.tb_rotate.setEnabled(True)

		self.tb_config = QAction(QIcon_load("preferences-system"), _("Configuration"), self)
		self.tb_config.triggered.connect(self.callback_configure)
		toolbar.addAction(self.tb_config)

		self.fx_box=fx_selector()
		self.fx_box.file_name_set_start("light_ray_")
		self.fx_box.file_name_set_end(".dat")
		self.fx_box.update()

		self.fx_box.cb.currentIndexChanged.connect(self.fx_box_changed)
		toolbar.addWidget(self.fx_box)

		self.hbox.addWidget(toolbar)

		enable_3d=inp_get_token_value(os.path.join(get_sim_path(),"config.inp") , "#gui_config_3d_enabled")
		if enable_3d==None:
			enable_3d="True"
		enable_3d=str2bool(enable_3d)

		if enable_3d==True:
			self.display=glWidget(self)
			self.update_ray_file()

			self.hbox.addWidget(self.display)
			#self.display.setMinimumSize(800, 600)

			self.timer=QTimer()
			self.timer.setSingleShot(True)
			self.timer.timeout.connect(self.timer_update)
			self.timer.start(5000)
		else:
			self.add_fallback()

		self.setLayout(self.hbox)
		global_object_register("display_recalculate",self.recalculate)
		global_object_register("display_set_selected_layer",self.set_selected_layer)



	def callback_xy(self):
		self.display.xy()

	def callback_yz(self):
		self.display.yz()

	def callback_xz(self):
		self.display.xz()

	def fx_box_changed(self):
		self.update_ray_file()
		self.display.update()
		#print("rod",self.display.ray_file)


	def tb_rotate_click(self):
		self.display.start_rotate()

	def timer_update(self):
		global open_gl_working

		open_gl_working=not self.display.failed

		if open_gl_working==True:
			print("OpenGL is working")
		else:
			print("OpenGL is not working going to fallback")
			self.hbox.removeWidget(self.display)
			self.display.deleteLater()
			self.display = None
			self.add_fallback()

			help_window().help_append(["warning.png",_("<big><b>OpenGL warning</b></big><br>It looks as if you don't have working 3D graphics acceleration on your computer.  gpvdm will therefore fallback to 2D mode. The model will still be fully functional, but not look quite so nice.")])

	def set_selected_layer(self,n):
		global open_gl_working
		if open_gl_working==True:
			self.display.selected_layer=n

	#This will reclaculate all the display elements in the display widget.
	def recalculate(self):
		self.fx_box.update()
		self.update_ray_file()
		self.display.graph_path=self.gl_cmp.slider.get_file_name()
		self.display.graph_z_max=self.gl_cmp.slider.z_max
		self.display.graph_z_min=self.gl_cmp.slider.z_min

		self.display.force_redraw()

	#def update(self):
#		print("recalculate")
	#	self.display.update()
	#	self.fx_box.update()

	def callback_configure(self):
		if self.gl_cmp.isVisible()==True:
			self.gl_cmp.hide()
		else:
			self.gl_cmp.show()

	#def callback_layer_editor(self):
	#	global_object_run("show_layer_editor")
