# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
import glob
from cal_path import get_device_lib_path
from inp_util import inp_search_token_value_multiline
from util_zip import zip_lsdir
from util_zip import archive_add_file

#qt
from PyQt5.QtWidgets import QMainWindow, QTextEdit, QAction, QApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QWidget,QSizePolicy,QHBoxLayout,QPushButton,QDialog,QTextEdit, QFileDialog, QToolBar, QLabel ,QMessageBox, QLineEdit,QVBoxLayout, QTableWidget, QAbstractItemView
from PyQt5.QtGui import QPixmap

#gui_util
from gui_util import tab_set_value
from gui_util import tab_add
from gui_util import tab_get_selected
from icon_lib import QIcon_load

from spinner import spinner

from error_dlg import error_dlg
from gui_util import yes_no_dlg

import traceback
import sys

from code_ctrl import enable_betafeatures

from report_error import report_error

def error_han(type, value, tback):
	if enable_betafeatures()==False:
		#formatted_lines = traceback.format_exc().splitlines()
		long_trace=traceback.format_exception(type, value, tback)
		long_trace=str("<br>".join(long_trace))
		trace=long_trace.replace("<br>","")
		trace=trace.replace(" ","")
		dialog=widget_error_han(long_trace,trace)
		dialog.exec_()
	sys.__excepthook__(type, value, tback)
	return True


class widget_error_han(QDialog):

	def __init__(self,long_error,error):

		self.error=error
		self.file_path=""
		QWidget.__init__(self)
		self.setWindowTitle(_("Error"))

		self.main_vbox=QVBoxLayout()
		self.setFixedSize(800,400)
		h_widget=QWidget()
		h_box=QHBoxLayout()
		h_widget.setLayout(h_box)
		image=QLabel()
		icon=QIcon_load("warning")
		image.setPixmap(icon.pixmap(icon.actualSize(QSize(48, 48))))
		h_box.addWidget(image)

		h_box.setAlignment(image,Qt.AlignTop)

		self.message = QTextEdit()
		help_text="<big><b>An error has occurred please report this error by clicking ok:<b></big><br><br>"
		help_text2="<br><br><big><b>It would also help if you e-mailed the error message to roderick.mackenzie@nottingham.ac.uk and described what you were doing with the model to make it crash.  Very often there is not enough information in bug reports alone to fix the problem.<br><br>All error reports are gratefully received.<br><br>Rod 5/9/16<b></big>"
		self.message.setText(help_text+long_error+help_text2)
		h_box.addWidget(self.message)

		self.main_vbox.addWidget(h_widget)

		button_widget=QWidget()
		self.main_vbox.addWidget(button_widget)

		self.label_reporting=QLabel(_("Reporting error...."))
		self.label_reporting.hide()

		self.spin=spinner()
		self.spin.hide()

		okButton = QPushButton("OK")
		cancelButton = QPushButton("Cancel")

		button_layout = QHBoxLayout()
		button_layout.addWidget(self.label_reporting)
		button_layout.addWidget(self.spin)

		button_layout.addStretch(1)
		button_layout.addWidget(okButton)
		button_layout.addWidget(cancelButton)
		button_widget.setLayout(button_layout)

		self.setLayout(self.main_vbox)

		okButton.clicked.connect(self.on_ok_clicked)
		cancelButton.clicked.connect(self.close_clicked)

	def error_reported(self,sucess):
		self.label_reporting.hide()
		self.spin.hide()
		if sucess==True:
			error_dlg(self,"I have reported the error, for more information e-mail roderick.mackenzie@nottingham.ac.uk")
			self.close()
		else:
			error_dlg(self,"I could not report the error please send the error message to roderick.mackenzie@nottingham.ac.uk")
			self.close()

	def close_clicked(self):
		result=yes_no_dlg(self,"Are you sure you don't want to report the error?  It would be really helpful if you did.")
		if result == True:
			self.close()
		else:
			self.on_ok_clicked()

	def on_ok_clicked(self):
		print("Reporting error....")
		self.label_reporting.show()
		self.spin.show()


		self.tx=report_error()
		self.tx.reported.connect(self.error_reported)
		self.tx.set_error(self.error)
		self.tx.start()

