# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
import sys
from scan_item import scan_item_add
from gui_util import dlg_get_text
import webbrowser
from util import fx_with_units
from icon_lib import QIcon_load
from scan_item import scan_remove_file

import i18n
_ = i18n.language.gettext

#inp
from inp_util import inp_search_token_value
from inp import inp_read_next_item

#matplotlib
import matplotlib
matplotlib.use('Qt5Agg')
import numpy as np
import matplotlib.pyplot as plt

#qt
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QWidget,QDialog,QVBoxLayout,QPushButton,QLabel,QSpinBox,QAction,QTabWidget,QTableWidget,QAbstractItemView
from PyQt5.QtGui import QPainter,QIcon


class fit_poly(QDialog):

	def __init__(self,x_min,x_max,data):
		QWidget.__init__(self)
		self.setWindowTitle(_("Poly fit")+"(https://www.gpvdm.com)")
		self.data=data
		self.ret_math=None
		self.ret=None
		self.interal_data=[]
		for i in range (0,self.data.y_len):		#x_nm= [x * 1e9 for x in self.data.y_scale]
			x=self.data.y_scale[i]
			#print(x_min,x,x_max)
			if x>=x_min and x<=x_max:
				self.interal_data.append((self.data.y_scale[i],self.data.data[0][0][i]))
				#print(self.data.y_scale[i],self.data.data[0][0][i])
			#frequency, = self.ax1.plot(x_nm,self.data.data[0][0][i], 'bo-', linewidth=3 ,alpha=1.0)
		#print(self.interal_data)
		self.main_vbox=QVBoxLayout()
		self.label = QLabel(_("Polynomial coefficients"))
		self.main_vbox.addWidget(self.label)
		self.sp = QSpinBox()
		self.main_vbox.addWidget(self.sp)
		self.button = QPushButton(_("Ok"), self)
		self.button.clicked.connect(self.callback_click_ok)
		self.main_vbox.addWidget(self.button)
		self.setLayout(self.main_vbox)
		ret=False

	def callback_click_ok(self,data):
		points = np.array(self.interal_data)	#[(1, 1), (2, 2), (3, 3), (7, 3), (9, 3)]
		# get x and y vectors
		x = points[:,0]
		y = points[:,1]

		# calculate polynomial
		terms=int(self.sp.value())
		self.ret = np.polyfit(x, y, terms)
		f = np.poly1d(self.ret)

		tot=""
		val=0
		for i in range(0,len(self.ret)):
			p=len(self.ret)-1-i
			tot=tot+str(self.ret[i])+"*pow(w,"+str(p)+")"+"+"
		tot=tot[:-1]
		self.ret_math=tot

		self.close()

	def run(self):
		self.exec_()

