# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
from gl_save import gl_save_list
from gl_save import gl_save_add

try:
	from OpenGL.GL import *
	from OpenGL.GLU import *
	from PyQt5 import QtOpenGL
	from PyQt5.QtOpenGL import QGLWidget
	from gl_color import set_color
	open_gl_ok=True
except:
	print("opengl error from gl_lib",sys.exc_info()[0])

import random
import numpy as np
from math import pi,acos,sin,cos

stars=[]

def gl_save_draw():
	save_list=gl_save_list()
	delta=6
	for i in range(0,len(save_list)):
		split=save_list[i].vec.split()
		if save_list[i].id=="box":
			w=float(split[0])
			h=float(split[1])
			d=float(split[2])
			r=float(split[3])
			g=float(split[4])
			b=float(split[5])
			alpha=float(split[6])
			box(save_list[i].x+delta,save_list[i].y,save_list[i].z,w,h,d,r,g,b,alpha)
		elif save_list[i].id=="photon":
			x=save_list[i].x
			z=save_list[i].z
			up=bool(split[0])
			draw_photon(x+delta,z,up)

def draw_stars():
	global stars
	if len(stars)==0:

		for i in range(0,5000):
			phi = random.uniform(0,2*pi)
			costheta = random.uniform(-1,1)
			theta = acos( costheta )
			r=70+random.uniform(0,300)
			x = r * sin( theta) * cos( phi )
			y = r * sin( theta) * sin( phi )
			z = r * cos( theta )
			color=random.uniform(0,1.0)
			r=color
			g=color
			b=color
			s=random.uniform(1,3)
			stars.append([x,y,z,r,g,b,s])

		stars.append([x,4,z,0.5,0.0,0.0,5])

	for i in range(0,len(stars)):
		glPointSize(stars[i][6])
		set_color(stars[i][3],stars[i][4],stars[i][5],"star")

		glBegin(GL_POINTS)
		glVertex3f(stars[i][0],stars[i][1],stars[i][2])
		#glVertex3f(-1.0,-1.0,0.0)
		glEnd()


def draw_grid():
	glLineWidth(1)

	set_color(0.5, 0.5, 0.5,"grid")

	start_x=-18.0
	stop_x=20.0
	n=int(stop_x-start_x)
	dx=1.0#(stop_x-start_x)/n
	pos=start_x
	glBegin(GL_LINES)
	for i in range(0,n+1):
		glVertex3f(start_x, 0.0, pos)
		glVertex3f(stop_x, 0.0, pos)
		pos=pos+dx


	start_z=-18.0
	stop_z=20.0
	dz=1.0#(stop_z-start_z)/n
	pos=start_z
	for i in range(0,n+1):
		glVertex3f(pos, 0, start_z)
		glVertex3f(pos, 0, stop_z)
		pos=pos+dz

	glEnd()



def draw_photon(x,z,up):
	gl_save_add("photon",x,-1.0,z,[int(up)])
	glLineWidth(3)
	length=0.9
	if up==True:
		set_color(0.0, 0.0, 1.0,"photon")
	else:
		set_color(0.0, 1.0, 0.0,"photon")


	wx=np.arange(0, length , 0.025)
	wy=np.sin(wx*3.14159*8)*0.2

	start_x=2.7
	stop_x=2.7-length

	glBegin(GL_LINES)
	for i in range(1,len(wx)):
		glVertex3f(x, start_x-wx[i-1], z+wy[i-1])
		glVertex3f(x, start_x-wx[i], z+wy[i])

	glEnd()

	if up==False:
		glBegin(GL_TRIANGLES)

		glVertex3f(x-0.1, stop_x,z)
		glVertex3f(x+0.1, stop_x ,z)
		glVertex3f(x,stop_x-0.1 ,z)

		glEnd()
	else:
		glBegin(GL_TRIANGLES)

		glVertex3f(x-0.1, start_x,z)
		glVertex3f(x+0.1, start_x ,z)
		glVertex3f(x,start_x+0.1 ,z)

		glEnd()

def box_lines(x,y,z,w,h,d):

	glLineWidth(10)

	set_color(1.0, 1.0, 0.0,"box_lines")

	glBegin(GL_LINES)

	#btm

	glVertex3f(x+0.0,y+0.0,z+0.0)
	glVertex3f(x+w,y+ 0.0,z+0.0)

	glVertex3f(x+w,y+ 0.0,z+0.0)
	glVertex3f(x+w,y+ 0.0,z+d)

	glVertex3f(x+w,y+ 0.0,z+d)
	glVertex3f(x+ 0.0, y+0.0,z+ d)


	#
	glVertex3f(x+0.0,y+h,z+0.0)
	glVertex3f(x+w,y+ h,z+0.0)


	glVertex3f(x+w,y+ h,z+0.0)
	glVertex3f(x+w,y+ h,z+d)

	glVertex3f(x+w,y+ h,z+d)
	glVertex3f(x+ 0.0, y+h,z+ d)

	#right

	glVertex3f(x+w,y,z)
	glVertex3f(x+w,y+ h,z)

	glVertex3f(x+w,y+ h,z)
	glVertex3f(x+w,y+ h,z+d)

	glVertex3f(x+w,y+ h,z+d)
	glVertex3f(x+w, y,z+d)

	#left

	glVertex3f(x,y,z)
	glVertex3f(x,y+ h,z)

	glVertex3f(x,y+ h,z)
	glVertex3f(x,y+ h,z+d)

	glVertex3f(x,y+ h,z+d)
	glVertex3f(x, y,z+d)


#
	glVertex3f(x,y,z+d)
	glVertex3f(x+w,y,z+d)

	glVertex3f(x+w,y,z+d)
	glVertex3f(x+w,y+h,z+d)

	glVertex3f(x+w,y+h,z+d)
	glVertex3f(x, y+h,z+d)


	#top
	glVertex3f(x,y+h,z)
	glVertex3f(x+w,y+ h,z)

	glVertex3f(x+w,y+ h,z)
	glVertex3f(x+w,y+ h,z+ d)

	glVertex3f(x+w,y+ h,z+ d)
	glVertex3f(x, y+h,z+ d)

	glEnd()

def box(x,y,z,w,h,d,r,g,b,alpha):
	gl_save_add("box",x,y,z,[w,h,d,r,g,b,alpha])
	red=r
	green=g
	blue=b


	#btm
	set_color(red,green,blue,"box_lines",alpha=alpha)

	glBegin(GL_QUADS)
	glVertex3f(x+0.0,y+0.0,z+0.0)
	glVertex3f(x+w,y+ 0.0,z+0.0)
	glVertex3f(x+w,y+ 0.0,z+d)
	glVertex3f(x+ 0.0, y+0.0,z+ d)
	glEnd()

	#back
	red=red*0.95
	green=green*0.95
	blue=blue*0.95

	set_color(red,green,blue,"box_lines",alpha=alpha)

	glBegin(GL_QUADS)
	glVertex3f(x+0.0,y+h,z+0.0)
	glVertex3f(x+w,y+ h,z+0.0)
	glVertex3f(x+w,y+ h,z+d)
	glVertex3f(x+ 0.0, y+h,z+ d)
	glEnd()

	#right
	red=red*0.95
	green=green*0.95
	blue=blue*0.95
	set_color(red,green,blue,"box_lines",alpha=alpha)

	glBegin(GL_QUADS)
	glVertex3f(x+w,y,z)
	glVertex3f(x+w,y+ h,z)
	glVertex3f(x+w,y+ h,z+d)
	glVertex3f(x+w, y,z+d)
	glEnd()

	#left
	red=red*0.95
	green=green*0.95
	blue=blue*0.95
	set_color(red,green,blue,"box_lines",alpha=alpha)

	glBegin(GL_QUADS)
	glVertex3f(x,y,z)
	glVertex3f(x,y+ h,z)
	glVertex3f(x,y+ h,z+d)
	glVertex3f(x, y,z+d)
	glEnd()

	#front
	red=r
	green=g
	blue=b

	set_color(red,green,blue,"box_lines",alpha=alpha)

	glBegin(GL_QUADS)
	glVertex3f(x,y,z+d)
	glVertex3f(x+w,y,z+d)
	glVertex3f(x+w,y+h,z+d)
	glVertex3f(x, y+h,z+d)
	glEnd()

	red=red*0.8
	green=green*0.8
	blue=blue*0.8

	#top
	set_color(red,green,blue,"box_lines",alpha=alpha)
	glBegin(GL_QUADS)
	glVertex3f(x,y+h,z)
	glVertex3f(x+w,y+ h,z)
	glVertex3f(x+w,y+ h,z+ d)
	glVertex3f(x, y+h,z+ d)
	glEnd()
