# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
from util import gpvdm_delete_file
#from global_objects import global_object_get
from plot_io import get_plot_file_info
from dat_file_class import dat_file

#qt
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QMenu,QAbstractItemView,QAction,QToolBar,QDialog,QVBoxLayout,QDialog,QWidget,QLineEdit

#cal_path
from icon_lib import QIcon_load
from cal_path import get_ui_path

from help import help_window

from error_dlg import error_dlg

from ref import get_ref_text
from gui_util import dlg_get_text
from gui_util import yes_no_dlg

from clone import clone_material
from clone import clone_spectra
from cal_path import get_base_material_path
from cal_path import get_base_spectra_path

from inp import inp_get_token_value
from util import isfiletype
from win_lin import desktop_open
from gpvdm_viewer import gpvdm_viewer

from util import wrap_text

COL_PATH = 0
COL_PIXBUF = 1
COL_IS_DIRECTORY = 2

import i18n
_ = i18n.language.gettext

#util
from util import latex_to_html

class gpvdm_open(QDialog):

	def __init__(self,path,show_inp_files=True,act_as_browser=True,big_toolbar=False):
		QWidget.__init__(self)
		self.act_as_browser=act_as_browser
		self.menu_new_material_enabled=False
		self.menu_new_spectra_enabled=False
		self.show_inp_files=show_inp_files
		self.show_directories=True
		self.file_path=""
		self.vbox=QVBoxLayout()
		self.setLayout(self.vbox)

		self.toolbar=QToolBar()
		if big_toolbar==True:
			self.toolbar.setIconSize(QSize(42, 42))
			self.toolbar.setToolButtonStyle( Qt.ToolButtonTextUnderIcon)
		#self.toolbar.setMaximumHeight(50)

		self.up = QAction(QIcon_load("go-up"), wrap_text(_("Back"),8), self)
		self.up.triggered.connect(self.on_up_clicked)
		self.toolbar.addAction(self.up)

		self.home = QAction(QIcon_load("user-home"), wrap_text(_("Home"),8), self)
		self.home.triggered.connect(self.on_home_clicked)
		self.toolbar.addAction(self.home)

		self.resize(800,500)
		self.path_widget=QLineEdit()
#		self.path_widget.setMinimumHeight(30)
#		self.path_widget.setStyleSheet("padding: 0; ")
		self.toolbar.addWidget(self.path_widget)



		self.setWindowTitle(_("Open file")+" https://www.gpvdm.com")
		self.setWindowIcon(QIcon_load("folder"))
#		self.listwidget=QListWidget()
#		self.listwidget.setSelectionMode(QAbstractItemView.ExtendedSelection)
#		self.listwidget.setStyleSheet("margin: 0; padding: 0; ")
		self.vbox.addWidget(self.toolbar)

		if self.act_as_browser==False:
			self.viewer=gpvdm_viewer(path,open_own_files=False)
		else:
			self.viewer=gpvdm_viewer(path)

		self.viewer.set_directory_view(True)
		self.vbox.addWidget(self.viewer)

		self.root_dir= path

		self.path_widget.setText(path)

		self.viewer.path_changed.connect(self.change_path)

		if self.act_as_browser==False:
			self.viewer.accept.connect(self.callback_accept)

		self.change_path()
		self.show()

	def callback_accept(self):
		self.file_path=self.viewer.file_path
		self.accept()

	def get_icon(self, name):
		return QIcon_load(name+"_file")


	def get_filename(self):
		return self.file_path


	def on_home_clicked(self, widget):
		self.viewer.path = self.root_dir
		self.change_path()


	def change_path(self):
		self.path_widget.setText(self.viewer.path)

		self.viewer.fill_store()
		sensitive = True
		print(self.viewer.path,self.root_dir)
		if self.viewer.path == self.root_dir:
			sensitive = False

		self.up.setEnabled(sensitive)

	def on_up_clicked(self, widget):
		self.viewer.set_path(os.path.dirname(self.viewer.path))
		self.change_path()
		self.viewer.fill_store()

