# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os

from scan_select import select_param
from token_lib import tokens
from scan_item import scan_items_get_list

from plot_io import plot_save_oplot_file
from scan_io import scan_push_to_hpc
from scan_io import scan_import_from_hpc
from cal_path import get_exe_command
from icon_lib import QIcon_load
from scan_item import scan_items_get_file
from scan_item import scan_items_get_token
from util import str2bool

from scan_item import scan_items_lookup_item
from inp import inp_get_token_value

import i18n
_ = i18n.language.gettext

#qt
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QWidget,QVBoxLayout,QToolBar,QTextEdit,QSizePolicy,QAction,QTabWidget,QTableWidget,QAbstractItemView
from PyQt5.QtGui import QPainter,QIcon

from gui_util import tab_add
from gui_util import tab_remove
from gui_util import tab_get_value

from gpvdm_select import gpvdm_select

from scan_select import select_param

from cal_path import get_sim_path

class matlab_editor(QWidget):

	def load(self):
		if os.path.isfile(self.file_name):
			f = open(self.file_name)
			lines = f.read()
			f.close()
			self.text.setText(lines)

	def callback_edited(self):
		lines=[]
		a = open(self.file_name, "w")
		a.write(self.text.toPlainText())#.split("\n")
		a.close()

	def callback_run(self):
		octave=inp_get_token_value(os.path.join(get_sim_path(),"config.inp"), "#matlab_interpreter")
		command=octave+" "+self.file_name+" "+self.exp_file_name+" "+self.fit_target
		print(command)
		old_path=os.getcwd()
		os.chdir(get_sim_path())
		os.system(command)
		os.chdir(old_path)

	def __init__(self,index):
		QWidget.__init__(self)

		self.index=index
		self.file_name=os.path.join(get_sim_path(),"fit_math"+str(self.index)+".inp")
		self.exp_file_name=os.path.join(get_sim_path(),"fit_data"+str(self.index)+".inp")
		self.fit_target=os.path.join(get_sim_path(),"fit_target"+str(self.index)+".inp")

		self.vbox=QVBoxLayout()

		toolbar=QToolBar()
		toolbar.setIconSize(QSize(48, 48))

		self.tb_run = QAction(QIcon_load("media-playback-start"), _("Run"), self)
		self.tb_run.triggered.connect(self.callback_run)
		toolbar.addAction(self.tb_run)

		self.vbox.addWidget(toolbar)

		self.text = QTextEdit()
		self.text.setFontPointSize(24)
		self.load()
		self.text.textChanged.connect(self.callback_edited)

		self.vbox.addWidget(self.text)


		self.setLayout(self.vbox)
