# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os

from scan_select import select_param
from token_lib import tokens
from scan_item import scan_items_get_list

from plot_io import plot_save_oplot_file
from scan_io import scan_push_to_hpc
from scan_io import scan_import_from_hpc
from cal_path import get_exe_command
from icon_lib import QIcon_load
from scan_item import scan_items_get_file
from scan_item import scan_items_get_token
from util import str2bool

from scan_item import scan_items_lookup_item
from gui_util import tab_move_down
from gui_util import tab_move_up

import i18n
_ = i18n.language.gettext

#qt
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QWidget,QVBoxLayout,QToolBar,QSizePolicy,QAction,QTabWidget,QTableWidget,QAbstractItemView, QMenuBar,QTableWidgetItem
from PyQt5.QtGui import QPainter,QIcon

from gui_util import tab_add
from gui_util import tab_remove
from gui_util import tab_get_value

from inp import inp_save_lines_to_file
from inp import inp_update_token_value
from inp import inp_load_file
from inp import inp_get_token_value

from gpvdm_select import gpvdm_select

from scan_select import select_param

from cal_path import get_sim_path
from QWidgetSavePos import QWidgetSavePos
from window_list import resize_window_to_be_sane
from measure_ribbon import measure_ribbon

class measure_tab(QWidget):

	def insert_row(self,i,file_name,position,output_token,math):
		self.tab.blockSignals(True)
		self.tab.insertRow(i)

		item = QTableWidgetItem(file_name)
		self.tab.setItem(i,0,item)

		item = QTableWidgetItem(position)
		self.tab.setItem(i,1,item)

		item = QTableWidgetItem(output_token)
		self.tab.setItem(i,2,item)

		item = QTableWidgetItem(math)
		self.tab.setItem(i,3,item)

		self.tab.blockSignals(False)

	def callback_show_list(self):
		self.select_param_window.update()
		self.select_param_window.show()

	def callback_add_item(self):
		self.insert_row(self.tab.rowCount(),_("File"),_("Position"),_("Output token"),_("Mathematical operation"))
		self.save_combo()

	def callback_delete_item(self):
		tab_remove(self.tab)
		self.save_combo()

	def save_combo(self):
		lines=[]

		print("write to",self.file_name)
		self.measure_name=inp_get_token_value(self.file_name, "#measure_name")


		lines.append("#measure_name")
		lines.append(str(self.measure_name))

		lines.append("#measure_enable")
		lines.append(str(self.measure_enable))

		lines.append("#measure_compile_to_vector")
		lines.append(str(self.compile_to_vector))

		for i in range(0,self.tab.rowCount()):
			lines.append("#measure_file_"+str(i))
			lines.append(str(tab_get_value(self.tab,i, 0)))
			lines.append("#measure_pos_"+str(i))
			lines.append(str(tab_get_value(self.tab,i, 1)))
			lines.append("#measure_token_"+str(i))
			lines.append(str(tab_get_value(self.tab,i, 2)))
			lines.append("#measure_math_"+str(i))
			lines.append(str(tab_get_value(self.tab,i, 3)))
		lines.append("#ver")
		lines.append("1.0")
		lines.append("#end")

		inp_save_lines_to_file(self.file_name,lines)


	def tab_changed(self):
		self.save_combo()


	def create_model(self):
		lines=[]
		self.tab.clear()
		self.tab.setColumnCount(4)
		self.tab.setSelectionBehavior(QAbstractItemView.SelectRows)
		self.tab.setHorizontalHeaderLabels([_("File"), _("Position"), _("Output token"),_("Mathematical operation")])
		self.tab.setColumnWidth(2, 200)
		self.tab.setColumnWidth(3, 200)

		lines=inp_load_file(self.file_name)
		if lines!=False:

			pos=0

			pos=pos+2		#skip over the name

			token=lines[pos]
			pos=pos+1

			self.measure_enable=str2bool(lines[pos])
			pos=pos+1


			token=lines[pos]
			pos=pos+1

			self.compile_to_vector=str2bool(lines[pos])
			pos=pos+1

			mylen=len(lines)
			while(1):
				temp=lines[pos]
				if temp=="#end" or temp=="#ver":
					break
				pos=pos+1

				f=lines[pos]
				pos=pos+1

				temp=lines[pos]
				pos=pos+1

				p=lines[pos]
				pos=pos+1

				temp=lines[pos]
				pos=pos+1

				output=lines[pos]
				pos=pos+1

				temp=lines[pos]
				pos=pos+1

				math=lines[pos]
				pos=pos+1

				self.insert_row(self.tab.rowCount(),f,p,output,math)

				if pos>mylen:
					break

	def __init__(self,index):
		QWidget.__init__(self)
		self.index=index
		lines=[]
		self.file_name=os.path.join(get_sim_path(),"measure"+str(self.index)+".inp")
		self.vbox=QVBoxLayout()

		toolbar=QToolBar()
		toolbar.setIconSize(QSize(32, 32))

		self.tb_save = QAction(QIcon_load("list-add"), _("Add"), self)
		self.tb_save.triggered.connect(self.callback_add_item)
		toolbar.addAction(self.tb_save)

		self.tb_save = QAction(QIcon_load("list-remove"), _("Minus"), self)
		self.tb_save.triggered.connect(self.callback_delete_item)
		toolbar.addAction(self.tb_save)

		self.tb_down = QAction(QIcon_load("go-down"), _("Move down"), self)
		self.tb_down.triggered.connect(self.callback_move_down)
		toolbar.addAction(self.tb_down)

		self.tb_up = QAction(QIcon_load("go-up"), _("Move up"), self)
		self.tb_up.triggered.connect(self.callback_move_up)
		toolbar.addAction(self.tb_up)

		self.vbox.addWidget(toolbar)

		self.tab = QTableWidget()
		self.tab.resizeColumnsToContents()

		self.tab.verticalHeader().setVisible(False)

		self.select_param_window=select_param()
		self.select_param_window.init(self.tab)
		self.select_param_window.set_save_function(self.save_combo)

		self.create_model()

		self.tab.cellChanged.connect(self.tab_changed)

		self.vbox.addWidget(self.tab)


		self.setLayout(self.vbox)

	def rename(self,tab_name):
		inp_update_token_value(self.file_name, "#measure_name", tab_name)

	def callback_move_down(self):
		tab_move_down(self.tab)

	def callback_move_up(self):
		tab_move_up(self.tab)
