# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
import os
#import shutil
import threading
import gobject
#import multiprocessing
#import glob
#import socket
#from time import sleep
from win_lin import running_on_linux
#import subprocess
#from util import gui_print_path

if running_on_linux()==True:
	import pyinotify
else:
	import win32file
	import win32con

	FILE_LIST_DIRECTORY = 0x0001

class _IdleObject(gobject.GObject):

	def __init__(self):
		gobject.GObject.__init__(self)

	def emit(self, *args):
		gobject.idle_add(gobject.GObject.emit,self,*args)

class _FooThread(threading.Thread, _IdleObject):
	__gsignals__ = {
		"file_changed": (
		gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
		}

	def __init__(self, *args):
		threading.Thread.__init__(self)
		_IdleObject.__init__(self)
		self.notifier=False

	def onChange(self,ev):
		if running_on_linux()==True:
			file_name=os.path.basename(ev.pathname)
		else:
			file_name=os.path.basename(ev)

		file_name=file_name.rstrip()
		self.thread_data[0]
		self.thread_data[0]=file_name
		print("thread: file changed")
		self.emit("file_changed")

	def set_watch_path(self,path,thread_data):
		self.watch_path=path
		self.thread_data=thread_data

	def run(self):
		if running_on_linux()==True:
			print("thread: start")
			wm = pyinotify.WatchManager()
			print("wathcing path",self.watch_path)
			ret=wm.add_watch(self.watch_path, pyinotify.IN_CLOSE_WRITE, self.onChange,False,False)
			print(ret)
			print("thread: start notifyer",self.notifier)
			self.notifier = pyinotify.Notifier(wm)
			try:
				while 1:
					self.notifier.process_events()
					if self.notifier.check_events():
						self.notifier.read_events()
			#self.notifier.loop()
			except:
				print("error in notify",sys.exc_info()[0])
		else:
			hDir = win32file.CreateFile (self.watch_path,FILE_LIST_DIRECTORY,win32con.FILE_SHARE_READ | win32con.FILE_SHARE_WRITE | win32con.FILE_SHARE_DELETE,None,win32con.OPEN_EXISTING,win32con.FILE_FLAG_BACKUP_SEMANTICS,None)

			while 1:
				results = win32file.ReadDirectoryChangesW (hDir,1024,True,
				win32con.FILE_NOTIFY_CHANGE_FILE_NAME |
				win32con.FILE_NOTIFY_CHANGE_DIR_NAME |
				win32con.FILE_NOTIFY_CHANGE_ATTRIBUTES |
				win32con.FILE_NOTIFY_CHANGE_SIZE |
				win32con.FILE_NOTIFY_CHANGE_LAST_WRITE |
				win32con.FILE_NOTIFY_CHANGE_SECURITY,
				None,
				None)

				for action, file in results:
					full_filename = os.path.join (self.watch_path, file)
					self.onChange(full_filename)


	def stop(self):
		print("thread: stop been called",threading.currentThread())
		if running_on_linux()==True:
			self.notifier.stop()
			self.notifier=False
			print("thread:I have shutdown the notifyer",threading.currentThread())
