# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os

from dump_io import dump_io
from tb_item_sim_mode import tb_item_sim_mode
from tb_item_sun import tb_item_sun

from code_ctrl import enable_betafeatures
from cal_path import get_css_path

#qt
from PyQt5.QtWidgets import QMainWindow, QTextEdit, QAction, QApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import QSize, Qt,QFile,QIODevice
from PyQt5.QtWidgets import QWidget,QSizePolicy,QVBoxLayout,QHBoxLayout,QPushButton,QDialog,QFileDialog,QToolBar,QMessageBox, QLineEdit, QToolButton
from PyQt5.QtWidgets import QTabWidget

#from ribbon_device import ribbon_device
from ribbon_database import ribbon_database
from ribbon_simulations import ribbon_simulations
from ribbon_configure import ribbon_configure
from ribbon_information import ribbon_information
from ribbon_home import ribbon_home
from icon_lib import QIcon_load

from about import about_dlg

from ribbon_cluster import ribbon_cluster
from css import css_apply

from status_icon import status_icon_stop
from global_objects import global_object_get
from server import server_get

from connect_to_cluster import connect_to_cluster

class ribbon(QTabWidget):
	def goto_page(self,page):
		self.blockSignals(True)
		for i in range(0,self.count()):
				if self.tabText(i)==page:
					self.setCurrentIndex(i)
					break
		self.blockSignals(False)

	def file(self):
		toolbar = QToolBar()
		toolbar.setToolButtonStyle( Qt.ToolButtonTextUnderIcon)
		toolbar.setIconSize(QSize(42, 42))

		self.home_new = QAction(QIcon_load("document-new"), _("New simulation").replace(" ","\n"), self)
		#self.home_new.setText(_("New\nsimulation"))
		toolbar.addAction(self.home_new)

		self.home_open = QAction(QIcon_load("document-open"), _("Open\nsimulation"), self)
		toolbar.addAction(self.home_open)

		self.home_export = QAction(QIcon_load("document-export"), _("Export\ndata"), self)
		toolbar.addAction(self.home_export)

		spacer = QWidget()
		spacer.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
		toolbar.addWidget(spacer)

		self.home_help = QAction(QIcon_load("internet-web-browser"), _("Help"), self)
		toolbar.addAction(self.home_help)

		return toolbar


	def update(self):
		#self.device.update()
		self.database.update()
		self.simulations.update()
		self.configure.update()
		self.information.update()
		self.home.update()

	def callback_about_dialog(self):
		dlg=about_dlg()
		dlg.exec_()

	def __init__(self):
		QTabWidget.__init__(self)
		self.cluster_tab=None
		self.setMaximumHeight(140)

		#self.setStyleSheet("QWidget {	background-color:cyan; }")

		self.myserver=server_get()

		self.holder=QWidget()
		self.hbox=QHBoxLayout()
		self.holder.setLayout(self.hbox)
		self.toolbar=QToolBar()
		self.toolbar.setIconSize(QSize(32, 32))

		self.about = QToolButton(self)
		self.about.setText(_("About"))
		self.about.pressed.connect(self.callback_about_dialog)

		self.cluster_button = QAction(QIcon_load("not_connected"), _("Connect to cluster"), self)
		self.cluster_button.triggered.connect(self.callback_cluster_connect)
		self.toolbar.addAction(self.cluster_button)

		self.hbox.addWidget(self.toolbar)
		self.hbox.addWidget(self.about)

		self.setCornerWidget(self.holder)

		w=self.file()
		self.addTab(w,_("File"))

		self.home=ribbon_home()
		self.addTab(self.home,_("Home"))

		self.simulations=ribbon_simulations()
		self.addTab(self.simulations,_("Simulations"))

		self.configure=ribbon_configure()
		self.addTab(self.configure,_("Configure"))

		#self.device=ribbon_device()
		#self.addTab(self.device,_("Device"))

		self.database=ribbon_database()
		self.addTab(self.database,_("Databases"))

		if enable_betafeatures()==True:
			self.tb_cluster=ribbon_cluster()
			self.addTab(self.tb_cluster,_("Cluster"))

		self.information=ribbon_information()
		self.addTab(self.information,_("Information"))

		#self.setStyleSheet("QWidget {	background-color:cyan; }")
		css_apply(self,"style.css")


	def callback_cluster_connect(self):
		dialog=connect_to_cluster()
		if dialog.exec_():
			self.cluster_tab=global_object_get("cluster_tab")
			global_object_get("notebook_goto_page")(_("Terminal"))
			if self.myserver.connect()==False:
				error_dlg(self,_("Can not connect to cluster."))

			self.tb_cluster.update()
			if self.myserver.cluster==True:
				self.cluster_button.setIcon(QIcon_load("connected"))
				status_icon_stop(True)
			else:
				status_icon_stop(False)
				self.cluster_button.setIcon(QIcon_load("not_connected"))

