# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os

from dump_io import dump_io
from tb_item_sim_mode import tb_item_sim_mode
from tb_item_sun import tb_item_sun

from code_ctrl import enable_betafeatures
from cal_path import get_css_path

#qt
from PyQt5.QtWidgets import QMainWindow, QTextEdit, QAction, QApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import QSize, Qt,QFile,QIODevice
from PyQt5.QtWidgets import QWidget,QSizePolicy,QVBoxLayout,QHBoxLayout,QPushButton,QDialog,QFileDialog,QToolBar,QMessageBox, QLineEdit, QToolButton
from PyQt5.QtWidgets import QTabWidget

from plot_gen import plot_gen
from info import sim_info
from win_lin import desktop_open

#windows
from scan import scan_class
from help import help_window
from gpvdm_open import gpvdm_open
from error_dlg import error_dlg
from server import server_get
from fit_window import fit_window
from cmp_class import cmp_class

from global_objects import global_object_run
from util import isfiletype
from icon_lib import QIcon_load

from cal_path import get_sim_path
from inp import inp_isfile

class ribbon_home(QToolBar):
	def __init__(self):
		QToolBar.__init__(self)
		self.myserver=server_get()

		self.setToolButtonStyle( Qt.ToolButtonTextUnderIcon)
		self.setIconSize(QSize(42, 42))

		self.scan_window=None
		self.fit_window=None

		self.undo = QAction(QIcon_load("edit-undo"), _("Undo"), self)
		self.addAction(self.undo)

		self.addSeparator()

		self.run = QAction(QIcon_load("media-playback-start"), _("Run simulation"), self)
		self.addAction(self.run)

		self.stop = QAction(QIcon_load("media-playback-pause"), _("Stop\nsimulation"), self)
		self.stop.triggered.connect(self.callback_simulate_stop)
		self.addAction(self.stop)

		self.addSeparator()

		self.scan = QAction(QIcon_load("scan"), _("Parameter\nscan"), self)
		self.scan.triggered.connect(self.callback_scan)
		self.addAction(self.scan)


		self.addSeparator()

		self.fit = QAction(QIcon_load("fit"), _("Fit\ndata"), self)
		self.fit.triggered.connect(self.callback_run_fit)
		self.addAction(self.fit)

		self.addSeparator()

		self.plot = QAction(QIcon_load("plot"), _("Plot\nFile"), self)
		self.plot.triggered.connect(self.callback_plot_select)
		self.addAction(self.plot)

		self.time = QAction(QIcon_load("plot_time"), _("Examine results\nin time domain"), self)
		self.time.triggered.connect(self.callback_examine)
		self.addAction(self.time)

		self.addSeparator()

		self.sun=tb_item_sun()
		self.sun.changed.connect(self.callback_sun)
		self.addWidget(self.sun)

		spacer = QWidget()
		spacer.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
		self.addWidget(spacer)

		self.help = QAction(QIcon_load("internet-web-browser"), _("Help"), self)
		self.addAction(self.help)



	def callback_sun(self):
		global_object_run("gl_force_redraw")

	def update(self):
		if self.scan_window!=None:
			del self.scan_window
			self.scan_window=None

		if self.fit_window!=None:
			del self.fit_window
			self.fit_window=None
		self.sun.update()

		if inp_isfile(os.path.join(get_sim_path(),"fit.inp"))==True:
			self.fit.setVisible(True)
		else:
			self.fit.setVisible(False)

	def setEnabled(self,val):
		self.undo.setEnabled(val)
		self.run.setEnabled(val)
		self.stop.setEnabled(val)
		self.scan.setEnabled(val)
		self.plot.setEnabled(val)
		self.time.setEnabled(val)
		self.sun.setEnabled(val)
		self.help.setEnabled(val)
		if enable_betafeatures()==True:
			self.fit.setEnabled(val)

	def callback_plot_select(self):
		help_window().help_set_help(["dat_file.png",_("<big>Select a file to plot</big><br>Single clicking shows you the content of the file")])

		dialog=gpvdm_open(get_sim_path(),show_inp_files=False)
		dialog.show_directories=False
		ret=dialog.exec_()
		if ret==QDialog.Accepted:
			file_name=dialog.get_filename()

			if os.path.basename(dialog.get_filename())=="sim_info.dat":
				self.sim_info_window=sim_info(dialog.get_filename())
				self.sim_info_window.show()
				return

			plot_gen([dialog.get_filename()],[],"auto")

			#self.plotted_graphs.refresh()
			#self.plot_after_run_file=dialog.get_filename()

	def callback_scan(self, widget):
		help_window().help_set_help(["scan.png",_("<big><b>The scan window</b></big><br> Very often it is useful to be able to systematically very a device parameter such as mobility or density of trap states.  This window allows you to do just that."),"list-add.png",_("Use the plus icon to add a new scan line to the list.")])
		#self.tb_run_scan.setEnabled(True)

		if self.scan_window==None:
			self.scan_window=scan_class(server_get())


		if self.scan_window.isVisible()==True:
			self.scan_window.hide()
		else:
			self.scan_window.show()

	def callback_examine(self, widget):
		help_window().help_set_help(["plot_time.png",_("<big><b>Examine the results in time domain</b></big><br> After you have run a simulation in time domain, if is often nice to be able to step through the simulation and look at the results.  This is what this window does.  Use the slider bar to move through the simulation.  When you are simulating a JV curve, the slider sill step through voltage points rather than time points.")])
		self.my_cmp_class=cmp_class()
		self.my_cmp_class.show()

		return
		ret=mycmp.init()
		if ret==False:
			error_dlg(self,_("Re-run the simulation with 'dump all slices' set to one to use this tool."))
			return

	def callback_simulate_stop(self):
		server_get().killall()

	def callback_run_fit(self, widget):
		if self.fit_window==None:
			self.fit_window=fit_window("fit")
			server_get().set_fit_update_function(self.fit_window.update)

		help_window().help_set_help(["fit.png",_("<big><b>Fit window</b></big><br> Use this window to fit the simulation to experimental data.")])
		if self.fit_window.isVisible()==True:
			self.fit_window.hide()
		else:
			self.fit_window.show()
