# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import sys
import os
from inp import inp_load_file
from token_lib import tokens
from cal_path import find_materials
from cal_path import get_materials_path
from util_zip import zip_lsdir
from cal_path import get_sim_path
#import shutil

check_list=[]

def scan_items_clear():
	global check_list
	check_list=[]

class scan_item:
	name=""
	token=""
	filename=""
	line=""

def scan_item_add(file_name,token,text_info,line):
	global check_list
	check_list.append(scan_item())
	listpos=len(check_list)-1
	text_info=text_info.replace("<sub>","")
	text_info=text_info.replace("</sub>","")
	check_list[listpos].name=os.path.join(os.path.splitext(file_name)[0],text_info)
	check_list[listpos].filename=file_name
	check_list[listpos].token=token
	check_list[listpos].line=line

def scan_items_populate_from_known_tokens():
	my_token_lib=tokens().get_lib()
	for i in range(0,len(my_token_lib)):
		if my_token_lib[i].file_name!="":
			scan_item_add(my_token_lib[i].file_name,my_token_lib[i].token,my_token_lib[i].info,1)

	#mat=find_materials()

	#for i in range(0,len(mat)):
	#	scan_remove_file(os.path.join(get_materials_path(),mat[i]))
	#	scan_item_add(os.path.join("materials",mat[i],"fit.inp"),"#wavelength_shift_alpha","Absorption spectrum wavelength shift",1)
	#	scan_item_add(os.path.join("materials",mat[i],"fit.inp"),"#n_mul","Refractive index spectrum multiplier",1)
	#	scan_item_add(os.path.join("materials",mat[i],"fit.inp"),"#alpha_mul","Absorption spectrum multiplier",1)

def scan_items_populate_from_files():
	name=os.path.join(get_sim_path(),"sim.gpvdm")
	if os.path.isfile(name)==True:
		file_list=zip_lsdir(name)

		for i in range(0,len(file_list)):
			if file_list[i].startswith("dos")==True and file_list[i].endswith(".inp")==True:
				scan_populate_from_file(file_list[i])

			if file_list[i].startswith("jv")==True and file_list[i].endswith(".inp")==True:
				scan_populate_from_file(file_list[i])
			#if my_token_lib[i].file_name!="":
			#	scan_item_add(my_token_lib[i].file_name,my_token_lib[i].token,my_token_lib[i].info,1)

		mat=find_materials()

		for i in range(0,len(mat)):
			scan_remove_file(os.path.join(get_materials_path(),mat[i]))
			scan_item_add(os.path.join("materials",mat[i],"fit.inp"),"#wavelength_shift_alpha","Absorption spectrum wavelength shift",1)
			scan_item_add(os.path.join("materials",mat[i],"fit.inp"),"#n_mul","Refractive index spectrum multiplier",1)
			scan_item_add(os.path.join("materials",mat[i],"fit.inp"),"#alpha_mul","Absorption spectrum multiplier",1)


def scan_item_save(file_name):
	global check_list
	f = open(file_name,'w')
	f.write(str(len(check_list))+"\n")
	for i in range(0,len(check_list)):
		f.write(check_list[i].name+"\n")
		f.write(check_list[i].filename+"\n")
		f.write(check_list[i].token+"\n")
		f.write(str(check_list[i].line)+"\n")
	f.close()

def scan_remove_file(file_name):
	global check_list
	new_list=[]
	for i in range(0,len(check_list)):
		if 	check_list[i].filename!=file_name:
			new_list.append(check_list[i])

	check_list=new_list


def scan_items_get_file(item):
	global check_list
	for i in range(0,len(check_list)):
		if check_list[i].name==item:
			return check_list[i].filename

	return "notknown"

def scan_items_get_token(item):
	global check_list
	for i in range(0,len(check_list)):
		if check_list[i].name==item:
			return check_list[i].token

	return "notknown"

def scan_items_lookup_item(filename,token):
	global check_list
	for i in range(0,len(check_list)):
		if check_list[i].filename==filename and check_list[i].token==token:
			return check_list[i].name

	return "notknown"

def scan_items_get_list():
	global check_list
	return check_list

def scan_items_index_item(item):
	global check_list
	for i in range(0,len(check_list)):
		if check_list[i].name==item:
			return i

	return -1

def scan_populate_from_file(filename):
	lines=[]
	lines=inp_load_file(filename)

	my_token_lib=tokens()

	for i in range(0, len(lines)):
		token=lines[i]
		if len(token)>0:
			if token[0]=="#":
				result=my_token_lib.find(token)
				if result!=False:
					if scan_items_index_item(token)==-1:
						scan_item_add(filename,token,result.info,1)

