# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
import os
from util import str2bool
from inp import inp_get_token_value
import threading
import multiprocessing
import time
from cal_path import get_exe_name
from cal_path import get_exe_args

from time import sleep
from win_lin import running_on_linux
import subprocess
from util import gui_print_path
from progress import progress_class

from cal_path import get_exe_command
from sim_warnings import sim_warnings
from inp_util import inp_search_token_value
from stat import *
from encrypt import encrypt
from encrypt import decrypt
from encrypt import encrypt_load
import i18n
_ = i18n.language.gettext
from cluster import cluster


from status_icon import status_icon_init
from status_icon import status_icon_run
from status_icon import status_icon_stop

import codecs
from gui_enable import gui_get

if gui_get()==True:
	from help import help_window
	from PyQt5.QtCore import pyqtSignal
	from PyQt5.QtWidgets import QWidget
	from PyQt5.QtCore import QTimer
	from process_events import process_events

from server_io import server_find_simulations_to_run

from workbook import gen_workbook
import time

from cal_path import get_sim_path
from gui_enable import gui_get
from job import job
import socket

my_server=False

from datetime import datetime


class node:
	ip=""
	load=""
	cpus=""

class job:
	name=""
	path=""
	ip=""
	args=""
	start=""
	stop=""
	cpus=1
	status=0

class base_server():
	def __init__(self):
		self.running=False
		self.progress_window=progress_class()
		self.stop_work=False
		self.cpus=multiprocessing.cpu_count()
		self.jobs=[]
		self.jobs_running=0
		self.jobs_run=0
		self.finished_jobs=[]

	def base_server_init(self,sim_dir):
		self.sim_dir=sim_dir



	def base_server_add_job(self,path,arg):
		j=job()
		j.path=path
		j.args=arg
		j.status=0
		j.name="job"+str(len(self.jobs))
		self.jobs.append(j)

	def check_warnings(self):
		message=""
		problem_found=False
		print(len(self.jobs))

		for i in range(0,len(self.jobs)):
			log_file=os.path.join(self.jobs[i].path,"log.dat")
			if os.path.isfile(log_file):
				f = open(log_file, "r")
				lines = f.readlines()
				f.close()
				found=""
				for l in range(0, len(lines)):
					lines[l]=lines[l].rstrip()
					if lines[l].startswith("error:") or lines[l].startswith("warning:"):
						#print("whoo",lines[l])
						found=found+lines[l]+"\n"
						problem_found=True
				if len(found)!=0:
					message=message+self.jobs[i].path+":\n"+found+"\n"
				else:
					message=message+self.jobs[i].path+":OK\n\n"
		if problem_found==False:
			message=""

		return message

	def print_jobs(self):
		print("server job list:")
		for i in range(0, len(self.jobs)):
			print("job",i,self.jobs[i].path,self.jobs[i].args,self.jobs[i].status)
		print("jobs running=",self.jobs_running,"jobs run=",self.jobs_run,"cpus=",self.cpus)

	def base_server_process_jobs(self):
		path=True
		while(path!=False):
			path,command=self.base_server_get_next_job_to_run(lock_file=True)
			if path!=False:
				cmd="cd "+path+";"
				cmd=cmd+command+" >/dev/null &\n"
				#print(cmd)
				os.system(cmd)

				self.progress_window.set_text(_("Running job ")+path)
				self.progress_window.set_fraction(float(self.jobs_run)/float(len(self.jobs)))

			else:
				return

	def remove_lock_files(self):
		ls=os.listdir(self.sim_dir)
		#print(">>>>>>",ls)
		for i in range(0, len(ls)):
			if ls[i][:4]=="lock" and ls[i][-4:]==".dat":
				del_file=os.path.join(self.sim_dir,ls[i])
				#print("delete file:",del_file)
				os.remove(del_file)

	def simple_run(self):
		self.stop_work=False
		self.remove_lock_files()

		self.base_server_process_jobs()

		while(1):
			ls=os.listdir(self.sim_dir)
			for i in range(0, len(ls)):
				if ls[i][:4]=="lock" and ls[i][-4:]==".dat":
					lock_file=ls[i]
					os.remove(os.path.join(self.sim_dir,lock_file))
					self.jobs_run=self.jobs_run+1
					self.jobs_running=self.jobs_running-1
			self.base_server_process_jobs()
			time.sleep(0.1)

			if self.jobs_run==len(self.jobs):
				self.remove_lock_files()
				break

	def base_server_get_next_job_to_run(self,lock_file=False):
		if (len(self.jobs)==0):
			return False,False

		for i in range(0, len(self.jobs)):
			if (self.jobs_running<self.cpus):
				if self.jobs[i].status==0:
					self.jobs[i].status=1
					self.jobs[i].start=str(datetime.now())
					self.jobs[i].ip=socket.gethostbyname(socket.gethostname())
					#for r in range(0,len(self.jobs)):
					#	print(self.jobs[i],self.args[i])

					self.jobs_running=self.jobs_running+1

					if lock_file==True:
						command_lock=" --lockfile "+os.path.join(self.sim_dir,"lock"+str(i)+".dat")
					else:
						command_lock=" --lock "+"lock"+str(i)

					full_command=get_exe_command()+command_lock+" "+self.jobs[i].args+" "+get_exe_args()

					return self.jobs[i].path,full_command

		return False,False


	def killall(self):
		self.stop_work=True
		if self.cluster==True:
			self.cluster_killall()
		else:
			if running_on_linux()==True:
				cmd = 'killall '+get_exe_name()
				os.system(cmd)
				print(cmd)
			else:
				cmd="taskkill /im "+get_exe_name()
				print(cmd)
				os.system(cmd)

if gui_get()==True:
	class server(QWidget,base_server,cluster):

		sim_finished = pyqtSignal()

		def __init__(self):
			QWidget.__init__(self)
			base_server.__init__(self)
			self.enable_gui=False
			self.callback_when_done=False
			self.display=False
			self.fit_update=None
			self.excel_workbook_gen_error=False
			status_icon_init()
			self.gui_update_time= time.time()
			self.timer=QTimer()

		def init(self,sim_dir):
			self.base_server_init(sim_dir)

			self.cluster_init()
			#self.cluster=str2bool(inp_get_token_value("server.inp","#cluster"))



		def set_terminal(self,terminal):
			self.terminal=terminal

		def set_display_function(self,display):
			self.display=display

		def set_fit_update_function(self,fit_update):
			self.fit_update=fit_update

		def gui_sim_start(self):
			help_window().hide()
			self.progress_window.start()
			status_icon_run(self.cluster)
			self.extern_gui_sim_start()

		def set_callback_when_done(self,proc):
			self.callback_when_done=proc

		def gui_sim_stop(self):
			text=self.check_warnings()
			self.progress_window.stop()
			help_window().show()
			status_icon_stop(self.cluster)

			help_window().help_set_help(["plot.png",_("<big><b>Simulation finished!</b></big><br>Click on the plot icon to plot the results")])

			if len(text)!=0:
				self.dialog=sim_warnings(text)

			if 	self.callback_when_done!=False:
				self.callback_when_done()
				self.callback_when_done=False

			if self.excel_workbook_gen_error==True:
				help_window().help_append(["warning.png",_("<big><b>Excel workbook error</b></big><br>I can't write new data to the file data.xlsx, I think you have are viewing it using another program.  Please close data.xlsx to enable me to write new data to it.")])

			self.jobs_update.emit()
			self.sim_finished.emit()

		def setup_gui(self,extern_gui_sim_start):
			self.enable_gui=True
			self.extern_gui_sim_start=extern_gui_sim_start


		def add_job(self,path,arg):
			if self.cluster==False:
				self.base_server_add_job(path,arg)
				self.jobs_update.emit()
			else:
				self.add_remote_job(path)
				self.send_dir(path,"")



		def start(self):
			self.excel_workbook_gen_error=False
			self.finished_jobs=[]
			self.progress_window.show()

			if self.enable_gui==True:
				self.gui_sim_start()

			self.running=True
			self.run_jobs()


		def run_jobs(self):
			self.stop_work=False
			if self.cluster==True:
				self.cluster_run_jobs()
			else:
				self.timer_start()

		def process_jobs(self):

			path=True
			while(path!=False):
				path,command=self.base_server_get_next_job_to_run()
				self.jobs_update.emit()
				if path!=False:
					if self.terminal.run(path,command)==True:
						time.sleep(0.1)
						return True
					else:
						return False
				else:
					return

		def stop(self):
			self.timer.stop()

			self.progress_window.set_fraction(0.0)

			self.gui_sim_stop()

			self.jobs=[]
			self.jobs_running=0
			self.jobs_run=0
			self.running=False

			if self.display!=False:

				self.display()

			#print(_("I have shut down the server."))

		def my_timer(self):
			#This is to give the QProcess timer enough time to update
			if self.terminal.test_free_cpus()!=0:
				self.process_jobs()


		def timer_start(self):
			self.timer.timeout.connect(self.my_timer)
			self.timer.start(100)

		def callback_dbus(self,data_in):
			if data_in.startswith("hex"):
				data_in=data_in[3:]
				data=codecs.decode(data_in, 'hex')
				data=data.decode('ascii')

				if data.startswith("lock"):
					if len(self.jobs)==0:
						print(_("I did not think I was running any jobs"))
						self.stop()
					else:
						if self.finished_jobs.count(data)==0:
							job=int(data[4:])
							self.jobs[job].status=2
							self.jobs[job].stop=str(datetime.now())

							self.finished_jobs.append(data)
							make_work_book=inp_get_token_value(os.path.join(get_sim_path(),"dump.inp"),"#dump_workbook")
							if make_work_book!=None:
								if str2bool(make_work_book)==True:
									if gen_workbook(self.jobs[job].path,os.path.join(self.jobs[job].path,"data.xlsx"))==False:
										self.excel_workbook_gen_error=self.excel_workbook_gen_error or True
							self.jobs_run=self.jobs_run+1
							self.jobs_running=self.jobs_running-1
							self.progress_window.set_fraction(float(self.jobs_run)/float(len(self.jobs)))

							if (self.jobs_run==len(self.jobs)):
								self.stop()

				elif (data=="pulse"):
					if len(self.jobs)==1:
						splitup=data.split(":")
						if len(splitup)>1:
							text=data.split(":")[1]
							self.progress_window.set_text(text)
						#self.progress_window.progress.set_pulse_step(0.01)
						self.progress_window.pulse()
				elif (data.startswith("enable_pulse")):
					splitup=data.split(":")
					if len(splitup)>1:
						value=str2bool(data.split(":")[1])
						self.progress_window.enable_pulse(value)
				elif (data.startswith("percent")):
					if len(self.jobs)==1:
						splitup=data.split(":")
						if len(splitup)>1:
							frac=float(data.split(":")[1])
							self.progress_window.set_fraction(frac)
				elif (data.startswith("text")):
					if len(self.jobs)==1:
						splitup=data.split(":")
						if len(splitup)>1:
							self.progress_window.set_text(data.split(":")[1])
				elif (data.startswith("fit_run")):
					elapsed_time = time.time() - self.gui_update_time
					if elapsed_time>5:

						self.gui_update_time=time.time()

						if self.fit_update!=None:
							self.fit_update()

def server_break():
	global my_server
	if my_server.stop_work==True:
		my_server.stop_work=False
		return True

	return False

def server_init():
	global my_server
	my_server=server()

def server_get():
	global my_server
	return my_server

