# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
from PyQt5.QtWidgets import QWidget,QVBoxLayout,QCheckBox,QDesktopWidget,QPushButton


class checkBoxInput(QWidget):
    def __init__(self, parentObject, parent=None):
        super(checkBoxInput, self).__init__(parent)
        self.resize(100,200)
        self.parentObject = parentObject

        layout = QVBoxLayout()
        self.b1 = QCheckBox("Title")
        self.b1.setChecked(self.parentObject.states[0])
        layout.addWidget(self.b1)

        self.b2 = QCheckBox("BlackBody")
        self.b2.setChecked(self.parentObject.states[1])
        layout.addWidget(self.b2)

        self.b3 = QCheckBox("Extraterrestrial")
        self.b3.setChecked(self.parentObject.states[2])
        layout.addWidget(self.b3)

        self.b4 = QCheckBox("Direct")
        self.b4.setChecked(self.parentObject.states[3])
        if parentObject.enable_earthattr:
            layout.addWidget(self.b4)

        self.b5 = QCheckBox("Diffuse")
        self.b5.setChecked(self.parentObject.states[4])
        self.b5.stateChanged.connect(lambda: self.b5)
        if parentObject.enable_earthattr:
            layout.addWidget(self.b5)

        self.b6 = QCheckBox("Total")
        self.b6.setChecked(self.parentObject.states[5])
        if parentObject.enable_earthattr:
            layout.addWidget(self.b6)
        if parentObject.enable_marsattr:
            layout.addWidget(self.b6)
        if parentObject.enable_venusattr:
            layout.addWidget(self.b6)

        self.b7 = QCheckBox("Legend")
        self.b7.setChecked(self.parentObject.states[6])
        layout.addWidget(self.b7)

        self.okButton = QPushButton('Apply')
        layout.addWidget(self.okButton)
        self.okButton.clicked.connect(self.getPreferences)


        self.setLayout(layout)
        self.center()

    def getPreferences(self):
        TitleState = self.b1.isChecked()
        BlackBodyState = self.b2.isChecked()
        ETState = self.b3.isChecked()
        DirectState = self.b4.isChecked()
        DiffuseState = self.b5.isChecked()
        TotalState = self.b6.isChecked()
        LegendState = self.b7.isChecked()
        self.parentObject.states = [TitleState, BlackBodyState, ETState, DirectState, DiffuseState, TotalState, LegendState]
        self.close()

    def center(self):
        qr = self.frameGeometry()
        cp = QDesktopWidget().availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

