# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os

from tab_base import tab_base

from PyQt5.QtWidgets import QTabWidget,QTextEdit,QWidget,QHBoxLayout
from PyQt5.QtCore import QProcess, Qt
from PyQt5.QtGui import QPalette,QColor,QFont

from QHTabBar import QHTabBar

import multiprocessing
import functools
from cpu_usage import cpu_usage
from win_lin import running_on_linux
from hpc import hpc_class
from code_ctrl import enable_betafeatures
from global_objects import global_object_register
from jobs import jobs_view
from server import server_get

from css import css_apply

class output_box(QTextEdit):
	def __init__(self,device_type):
		QTextEdit.__init__(self)
		self.font = QFont()
		self.font.setFamily('Monospace')
		self.font.setStyleHint(QFont.Monospace)
		self.font.setFixedPitch(True)
		self.font.setPointSize(int(12))

		self.setFont(self.font)
		pal = QPalette()
		bgc = QColor(0, 0, 0)
		pal.setColor(QPalette.Base, bgc)
		textc = QColor(230, 230, 230)
		pal.setColor(QPalette.Text, textc)
		self.setPalette(pal)
		self.device_type=device_type
		self.setAcceptRichText(False)
		self.setContextMenuPolicy(Qt.NoContextMenu)
		#self.setOpenLinks(False)
		self.setReadOnly(True)
		self.setUndoRedoEnabled(False)

	def add_text(self,data):
		self.setUpdatesEnabled(False);
		cursor = self.textCursor()
		cursor.movePosition(cursor.End,cursor.MoveAnchor)
		self.setTextCursor(cursor)
		cursor.insertHtml(data)
		self.ensureCursorVisible()
		self.setUpdatesEnabled(True)

class tab_terminal(QWidget,tab_base):

	def __init__(self):
		QWidget.__init__(self)
		self.tab=QTabWidget()
		css_apply(self.tab,"style_h.css")
		self.vbox=QHBoxLayout()
		self.vbox.addWidget(self.tab)
		self.usage=cpu_usage()
		self.vbox.addWidget(self.usage)
		self.setLayout(self.vbox)
		self.my_server=server_get()

	def dataReady(self,i):
		#cursor = self.terminals[i].textCursor()
		#cursor.movePosition(cursor.End,cursor.MoveAnchor)
		#self.terminals[i].setTextCursor(cursor)
		r=self.process[i].readAll()
		data=str(r,'utf-8',errors='ignore')

		#cursor.insertHtml(data)
		self.terminals[i].add_text(data)

	def list_cpu_state(self):
		for i in range(0,self.cpus):
			if self.process[i].state()==QProcess.NotRunning:
				print(i,"0")
			else:
				print(i,"1")

	def run(self,path,command):
		for i in range(0,self.cpus):

			if self.process[i].state()==QProcess.NotRunning:
				cursor = self.terminals[i].textCursor()
				self.terminals[i].clear()
				cursor.insertHtml(_("Running: ")+command+"<br>")
				self.process[i].setWorkingDirectory(path)

				if running_on_linux()==False:
					if command.count(".exe")>0:
						command="\""+command
						command=command.replace(".exe",".exe\"",1)

				print("exe command=",command)
				self.process[i].start(command)
				return True

		print(_("I could not find a free cpu to run the command on"))
		return False

	def test_free_cpus(self):
		ret=0
		for i in range(0,self.cpus):

			if self.process[i].state()==QProcess.NotRunning:
				ret=ret+1

		return ret

	def init(self):
		self.cpus=multiprocessing.cpu_count()

		self.tab.setTabsClosable(True)
		self.tab.setMovable(True)
		self.tab.setTabBar(QHTabBar())
		self.tab.setTabPosition(QTabWidget.West)


		self.terminals=[]
		self.process=[]
		for i in range(0,self.cpus):
			term=output_box("local_cpu")

			proc=QProcess(self)
			proc.readyRead.connect(functools.partial(self.dataReady,i))
			self.process.append(proc)
			self.terminals.append(term)
			self.tab.addTab(term,_("CPU")+" "+str(i))


		self.cluster_output=output_box("cluster_node")
		self.tab.addTab(self.cluster_output,_("Cluster"))

		self.jview=jobs_view()
		#self.jview.load_data(self.myserver.cluster_jobs)
		self.tab.addTab(self.jview,"Jobs list")

		if enable_betafeatures()==True:
			self.cluster=hpc_class()
			self.tab.addTab(self.cluster,_("Nodes"))
			global_object_register("cluster_tab",self.cluster)


		self.my_server.new_message.connect(self.data_from_cluster)

	def data_from_cluster(self,data):
		#self.cluster_output.add_text(data+"\n")
		print(data+"\n")

	def help(self):
		my_help_class.help_set_help(["utilities-terminal.png","<big><b>The terminal window</b></big>\nThe model will run in this window.  You can also use it to enter bash commands."])

