# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import math
from layer_widget import layer_widget
import os
from cal_path import get_materials_path
from inp_util import inp_search_token_value
from util import str2bool
from tab_base import tab_base
from epitaxy import epitaxy_get_layers
from epitaxy import epitaxy_get_width
from epitaxy import epitaxy_get_mat_file
from epitaxy import epitaxy_get_electrical_layer
from help import help_window
from epitaxy import epitaxy_get_pl_file
from epitaxy import epitaxy_get_name
from display import display_widget
from win_lin import running_on_linux
from code_ctrl import enable_webupdates
from update import update_thread

from PyQt5.QtWidgets import QWidget,QHBoxLayout,QSplitter
from PyQt5.QtCore import Qt

from ribbon_device import ribbon_device

import i18n
_ = i18n.language.gettext

from gpvdm_viewer import gpvdm_viewer
from cal_path import get_sim_path
from server import server_get

class tab_view(QWidget,tab_base):


	def __init__(self):
		QWidget.__init__(self)

		hbox=QHBoxLayout(self)
		mainLayout = QSplitter(Qt.Horizontal)
		self.viewer=gpvdm_viewer(get_sim_path())
		self.viewer.set_directory_view(True)
		self.viewer.set_back_arrow(True)
		self.viewer.set_multi_select()

		hbox.addWidget(self.viewer)

		self.setLayout(hbox)
		server_get().sim_finished.connect(self.refresh)

	def refresh(self):
		self.viewer.fill_store()

	def help(self):
		help_window().help_set_help(["device.png",_("<big><b>The device structure tab</b></big>\n Use this tab to change the structure of the device, the layer thicknesses and to perform optical simulations.  You can also browse the materials data base and  edit the electrical mesh.")])

	def got_help(self,data):
		if data!="":
			help_window().help_append(["star.png",_("<big><b>Update available!</b></big><br>"+data)])
