# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
from inp import inp_load_file
from inp import inp_save

from cal_path import get_sim_path
from inp import inp_read_next_item
from PyQt5.QtWidgets import QWidget, QDesktopWidget

def resize_window_to_be_sane(window,x,y):
	shape=QDesktopWidget().screenGeometry()
	w=shape.width()*x
	h=shape.height()*y
	window.resize(w,h)

class window_item:
	name=""
	x=0.0
	y=0.0

wlist=[]

def wpos_set_window(window,name):
	#print("set")
	global wlist
	for i in range(0,len(wlist)):
		if wlist[i].name==name:
			shape=QDesktopWidget().screenGeometry()

			desktop_w=shape.width()
			desktop_h=shape.height()

			w=window.width()
			h=window.height()

			x=int(wlist[i].x)
			y=int(wlist[i].y)
			if (x+w>desktop_w) or x<0:
				x=desktop_w/2-w/2
				#print("Reset with",x)
			if (y+h>desktop_h) or y<0:
				y=desktop_h/2-h/2
				#print("Reset height",y)
			window.move(x,y)
			break

def wpos_dump():
	for i in range(0,len(wlist)):
		print(wlist[i].name,wlist[i].x,wlist[i].y)

def wpos_update(window,name):
	global wlist
	found=False
	x=window.x()
	y=window.y()
	#print(x,y)
	for i in range(0,len(wlist)):
		if wlist[i].name==name:
			found=True
			wlist[i].x=x
			wlist[i].y=y
			break

	if found==False:
		a=window_item()
		a.name=name
		a.x=x
		a.y=y
		wlist.append(a)

def wpos_load():
	#print("load")
	global wlist
	wlist=[]
	lines=[]
	pos=0
	lines=inp_load_file("window_list2.inp")
	if lines!=False:
		while(1):
			token,name,pos=inp_read_next_item(lines,pos)
			if token=="#end" or token=="#ver":
				break

			token,x,pos=inp_read_next_item(lines,pos)

			token,y,pos=inp_read_next_item(lines,pos)

			a=window_item()
			a.name=name
			a.x=float(x)
			a.y=float(y)
			wlist.append(a)


def wpos_save():
	#print("save")
	global wlist
	lines=[]

	for i in range(0,len(wlist)):
		lines.append("#window_name_"+wlist[i].name)
		lines.append(wlist[i].name)
		lines.append("#window_x_"+wlist[i].name)
		lines.append(str(wlist[i].x))
		lines.append("#window_y_"+wlist[i].name)
		lines.append(str(wlist[i].y))

	lines.append("#ver")
	lines.append("1.0")
	lines.append("#end")

	ret=inp_save("window_list2.inp",lines)
	return ret
