// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <sim.h>
#include <dump.h>
#include <buffer.h>
#include <util.h>
#include <lang.h>
#include <i.h>
#include <exp.h>
#include <dos.h>
#include <memory.h>



void dump_1d_slice(struct simulation *sim,struct device *in,char *out_dir)
{
int x;
int y;
int z;
long double ***temp_3d;
int band;
char name[100];
char temp[200];
gdouble Vexternal=get_equiv_V(sim,in);
struct buffer buf;
buffer_init(&buf);


	buffer_add_dir(sim,out_dir);

	cal_J_drift_diffusion(in);
	malloc_3d_gdouble(in, &temp_3d);
	buffer_malloc(&buf);
	sprintf(name,"%s","Jn_drift.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Drift current density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron current density (drift)"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"A m^{-2}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Jn_drift);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Jn_diffusion.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Diffusion current density"),_("position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron current density (diffusion)"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"A m^{-2}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Jn_diffusion);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Jp_drift.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Drift current density"),_("position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Hole current density (drift)"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"A m^{-2}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Jp_drift);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Jp_diffusion.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Diffusion current density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Hole current density (diffusion)"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"A m^{-2}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Jp_diffusion);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Ec.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("LUMO"),_("position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron Energy"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"eV");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Band structure"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Ec);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Ev.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("HOMO"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron Energy"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"eV");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Band structure"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Ev);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Tl.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Lattice temperature"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Temperature"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"K");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Tl);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Te.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Electron temperature"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Temperature"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"K");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Te);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Th.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Hole temperature"),_("position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Temperature"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"K");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Te);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Nad.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Doping"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Doping density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Charge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Nad);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Eg.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Band gap"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron Energy"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"eV");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Band structure"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Eg);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Fn.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Electron quasi Fermi-level"),_("position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron Energy"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"eV");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Band structure"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Fn);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Fp.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Hole quasi Fermi-level"),_("position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron Energy"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"eV");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Band structure"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Fp);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","phi.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Position"),_("Potential"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Potential"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"V");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Band structure"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->phi);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","dphi.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	strcpy(buf.title,_("Potential"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Potential"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"V");
	strcpy(buf.section_one,_("Change in 1D position space output"));
	strcpy(buf.section_two,_("Band structure"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->phi);
	three_d_add_gdouble(in, temp_3d, in->phi_save);
	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Jn.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Current density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron current density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"A m^{-2}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints+2;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	//printf("%Le %Le\n",in->Jnleft[0][0],in->Jnright[0][0]);
	//getchar();
	buffer_add_3d_device_data_including_boundaries(sim,&buf,in,  in->Jn,in->Jnleft[0][0]*Q,in->Jnright[0][0]*Q);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","Jp.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Current density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Hole current density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"A m^{-2}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints+2;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data_including_boundaries(sim,&buf,in,  in->Jp,in->Jpleft[0][0]*Q,in->Jpright[0][0]*Q);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Jn_x.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Electron current density x"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron current density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"A m^{-2}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Jn_x);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","Jp_x.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Electron current density x"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Hole current density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"A m^{-2}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Jp_x);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Jn_plus_Jp.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Total current density (Jn+Jp)"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Total current density (Jn+Jp)"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"A m^{-2}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	//sprintf(temp,"%Le %Le\n",in->ymesh[0]-in->ymesh[1]/2,get_J_left(in));

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->Jp);
	three_d_add_gdouble(in, temp_3d, in->Jn);
	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	//sprintf(temp,"%Le %Le\n",in->ymesh[in->ymeshpoints-1]-in->ymesh[1]/2,get_J_right(in));
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Jp_drift_plus_diffusion.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Total current density (Jn+Jp)"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Total current density (Jn+Jp)"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"A m^{-2}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Transport"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->Jp_drift);
	three_d_add_gdouble(in, temp_3d, in->Jp_diffusion);
	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Fi.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Equlibrium Fermi-level"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Energy"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"eV");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Band structure"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Fi);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","epsilon_r.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Relative permittivity"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Relative permittivity"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"au");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Material parameters"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->epsilonr);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","mu_n.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Electron mobility"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron mobility"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{2} V^{-1} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Material parameters"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->mun);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","mu_p.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Hole mobility"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Hole mobility"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{2} V^{-1} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Material parameters"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->mup);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","mu_n_ft.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s","%s mu0*nf/nall",_("Average electron mobility"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Mobility"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{2} V^{-1} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Material parameters"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	x=0;
	y=0;
	z=0;

	for (y=0;y<in->ymeshpoints;y++)
	{
		sprintf(temp,"%Le %Le\n",in->ymesh[y],in->mun[z][x][y]*in->n[z][x][y]/(in->nt_all[z][x][y]+in->n[z][x][y]));
		buffer_add_string(&buf,temp);
	}
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","mu_p_ft.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s mu0*nf/nall",_("Average electron mobility"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Mobility"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{2} V^{-1} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Material parameters"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	for (y=0;y<in->ymeshpoints;y++)
	{
		sprintf(temp,"%Le %Le\n",in->ymesh[y],in->mup[z][x][y]*in->p[z][x][y]/(in->pt_all[z][x][y]+in->p[z][x][y]));
		buffer_add_string(&buf,temp);
	}
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","nf.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Free electron carrier density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Carrier density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Material parameters"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints+2;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	//buffer_add_3d_device_data(sim,&buf,in,  in->n);
	buffer_add_3d_device_data_including_boundaries(sim,&buf,in,  in->n,in->l_electrons,in->r_electrons);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","pf.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Free hole carrier density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Carrier density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Material parameters"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints+2;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	//buffer_add_3d_device_data(sim,&buf,in,  in->p);
	buffer_add_3d_device_data_including_boundaries(sim,&buf,in,  in->p,in->l_holes,in->r_holes);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","nt.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Trapped electron carrier density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Carrier density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Charge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->nt_all);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","pt.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Trapped hole carrier density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Carrier density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Charge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->pt_all);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","p.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Total hole density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Carrier density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Charge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->p);
	three_d_add_gdouble(in, temp_3d, in->pt_all);
	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","n.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Total electron density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Carrier density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Charge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->n);
	three_d_add_gdouble(in, temp_3d, in->nt_all);
	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","dn.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,_("Change in electron population"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Carrier density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Chaerge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->n);
	three_d_add_gdouble(in, temp_3d, in->nt_all);
	three_d_sub_gdouble(in, temp_3d, in->nf_save);
	three_d_sub_gdouble(in, temp_3d, in->nt_save);
	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","charge.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Total charge"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Carrier density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Charge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->p);
	three_d_sub_gdouble(in, temp_3d, in->n);
	three_d_add_gdouble(in, temp_3d, in->pt_all);
	three_d_sub_gdouble(in, temp_3d, in->nt_all);
	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","dcharge.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Change in total charge"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Carrier density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Charge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->p);
	three_d_sub_gdouble(in, temp_3d, in->n);
	three_d_add_gdouble(in, temp_3d, in->pt_all);
	three_d_sub_gdouble(in, temp_3d, in->nt_all);

	three_d_sub_gdouble(in, temp_3d, in->pt_save);
	three_d_add_gdouble(in, temp_3d, in->nf_save);
	three_d_sub_gdouble(in, temp_3d, in->pf_save);
	three_d_add_gdouble(in, temp_3d, in->nt_save);

	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","dp.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Change in hole population"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Carrier density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Charge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->p);
	three_d_add_gdouble(in, temp_3d, in->pt_all);
	three_d_sub_gdouble(in, temp_3d, in->pf_save);
	three_d_sub_gdouble(in, temp_3d, in->pt_save);

	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","dnt.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Excess electron density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Charge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->nt_all);
	three_d_sub_gdouble(in, temp_3d, in->nt_save);
	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","dpt.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Excess hole density"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Hole density"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Charge density"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);

	three_d_set_gdouble(in, temp_3d, 0.0);
	three_d_add_gdouble(in, temp_3d, in->pt_all);
	three_d_sub_gdouble(in, temp_3d, in->pt_save);
	buffer_add_3d_device_data(sim,&buf,in, temp_3d);

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Gn.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Free electron generation rate"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Generation rate"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Gn);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Gp.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Free hole generation rate"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Generation rate"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Gp);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","Photon_gen.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Photon generation rate"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Photon generation rate"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Photon_gen);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","Rn_srh.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("SRH electron recombination rate"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Recombination rate"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Rn_srh);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","Rp_srh.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("SRH hole recombination rate"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Recombination rate"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Rp_srh);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","R_free.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Free electron-hole recombination rate"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Recombination rate"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->Rfree);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);
/*
	buffer_malloc(&buf);
	sprintf(name,"%s","fsrhn.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Trap fermi level"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron Fermi level"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"eV");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	for (y=0;y<in->ymeshpoints;y++)
	{
		sprintf(temp,"%Le ",in->ymesh[y]);
		buffer_add_string(&buf,temp);
		for (band=0;band<in->srh_bands;band++)
		{
			sprintf(temp,"%Le %Le ",in->Fnt[z][x][y][band],-in->phi[z][x][y]-in->Xi[z][x][y]+dos_srh_get_fermi_n(in,in->n[z][x][y], in->p[z][x][y],band,in->imat[z][x][y],in->Te[z][x][y]));
			buffer_add_string(&buf,temp);
		}
		buffer_add_string(&buf,"\n");

	}
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","fsrhh.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Trap fermi level"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Electron Fermi level"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"eV");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	for (y=0;y<in->ymeshpoints;y++)
	{
		sprintf(temp,"%Le ",in->ymesh[y]);
		buffer_add_string(&buf,temp);
		for (band=0;band<in->srh_bands;band++)
		{
			sprintf(temp,"%Le %Le ",in->Fpt[z][x][y][band],-in->phi[z][x][y]-in->Xi[z][x][y]-in->Eg[z][x][y]-dos_srh_get_fermi_p(in,in->n[z][x][y], in->p[z][x][y],band,in->imat[z][x][y],in->Th[z][x][y]));
			buffer_add_string(&buf,temp);
		}
		buffer_add_string(&buf,"\n");

	}
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);
*/

	buffer_malloc(&buf);
	sprintf(name,"%s","imat.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Material number"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Number"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"au");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Model"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	for (y=0;y<in->ymeshpoints;y++)
	{
		sprintf(temp,"%Le %d\n",in->ymesh[y], in->imat[z][x][y]);
		buffer_add_string(&buf,temp);
	}
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","Efield.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Material number"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Number"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"au");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Band structure"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	gdouble deriv=0.0;
	for (y=in->ymeshpoints-1;y>1;y--)
	{

		deriv= -(in->phi[z][x][y]-in->phi[z][x][y-1])/(in->ymesh[y]-in->ymesh[y-1]);
		sprintf(temp,"%Le %Le\n",in->ymesh[y], deriv);
		buffer_add_string(&buf,temp);
	}

	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","pf_to_nt.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Free hole to trapped electron"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Rate"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->ntrap_to_p);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	buffer_malloc(&buf);
	sprintf(name,"%s","nf_to_pt.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Free electron to trapped hole"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Rate"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->ptrap_to_n);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","prelax.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Hole relaxation rate"),_("position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Rate"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in,  in->prelax);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);

	buffer_malloc(&buf);
	sprintf(name,"%s","nrelax.dat");
	buf.y_mul=1e9;
	buf.x_mul=1e9;
	buf.z_mul=1e9;
	sprintf(buf.title,"%s - %s",_("Electron relaxation rate"),_("Position"));
	buffer_set_graph_type(&buf,in);
	strcpy(buf.x_label,_("x-position"));
	strcpy(buf.y_label,_("y-position"));
	strcpy(buf.z_label,_("z-position"));
	strcpy(buf.data_label,_("Rate"));
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"nm");
	strcpy(buf.z_units,"nm");
	strcpy(buf.data_units,"m^{-3} s^{-1}");
	strcpy(buf.section_one,_("1D position space output"));
	strcpy(buf.section_two,_("Recombination"));
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.time=in->time;
	buf.Vexternal=Vexternal;
	buf.x=in->xmeshpoints;
	buf.y=in->ymeshpoints;
	buf.z=in->zmeshpoints;
	buffer_add_info(sim,&buf);
	buffer_add_3d_device_data(sim,&buf,in, in->nrelax);
	buffer_dump_path(sim,out_dir,name,&buf);
	buffer_free(&buf);


	free_3d_gdouble(in, temp_3d);
}
