// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include "util.h"
#include "const.h"
#include "light.h"
#include "device.h"
#include "const.h"
#include "dump.h"
#include "config.h"
#include "inp.h"
#include "util.h"
#include "hard_limit.h"
#include "epitaxy.h"
#include "lang.h"
#include "log.h"

static int unused __attribute__((unused));


void light_init_mesh(struct simulation *sim,struct light *in)
{
printf_log(sim,"init: mesh\n");
	int i;
	gdouble pos=0.0;
	pos=in->dx;

	int layer=0;
	gdouble layer_end=in->thick[layer];

	for (i=0;i<in->points;i++)
	{
		in->x[i]=pos;
		in->layer_end[i]=layer_end-pos;
		in->layer[i]=layer;
		if (in->device_start_layer>=layer) in->device_start_i=i;

		pos+=in->dx;

		if (pos>layer_end)
		{

			//do
			//{
			if (layer<(in->layers-1))
			{
				layer++;
			//}while(in->thick[layer]==0.0);

				layer_end=layer_end+in->thick[layer];
			}
		}

	}
	in->device_start_i++;

	in->dl=(in->lstop-in->lstart)/((gdouble)in->lpoints);

	pos=in->lstart;
	for (i=0;i<in->lpoints;i++)
	{
		in->l[i]=pos;
		pos+=in->dl;
	}

	int ii;

	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			in->alpha[i][ii]=inter_get_noend(&(in->mat[in->layer[ii]]),in->l[i]);
			in->alpha0[i][ii]=in->alpha[i][ii];
			in->n[i][ii]=inter_get_noend(&(in->mat_n[in->layer[ii]]),in->l[i]);

		}
	}

	light_calculate_complex_n(in);


	for (i=0;i<in->lpoints;i++)
	{
		in->sun_norm[i]=inter_get_noend(&(in->sun_read),in->l[i]);
	}

	gdouble tot=0.0;
	for  (i=0;i<in->lpoints;i++)
	{
		tot+=in->dl*in->sun_norm[i];
	}

	for  (i=0;i<in->lpoints;i++)
	{
		in->sun_norm[i]/=tot;
	}


}
