// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include "sim.h"
#include "inp.h"
#include "fx.h"
#include <cal_path.h>

static int unused __attribute__((unused));

static long double *fx_mesh;
static int mesh_len=0;
static int mesh_pos=0;

void fx_mesh_save(struct simulation *sim)
{
	int i;
	FILE *out;
	out=fopen("fxmesh_save.dat","w");

	if (out==NULL)
	{
		ewe(sim,"can not save fx mesh file\n");
	}

	fprintf(out, "%d\n",mesh_len);

	for (i=0;i<mesh_len;i++)
	{
		fprintf(out, "%Le\n",fx_mesh[i]);
	}

	fprintf(out, "#ver\n");
	fprintf(out, "1.0\n");
	fprintf(out, "#end\n");

	fclose(out);
}

void fx_load_mesh(struct simulation *sim,struct device *in,int number)
{
	struct inp_file inp;
	char mesh_file[200];

	long double start=0.0;
	long double stop=0.0;
	long double points=0.0;
	long double mul=0.0;

	long double dfx=0.0;

	int buffer_len=2000;
	long double fx=0.0;
	int ii=0;
	char temp[200];
	fx_mesh=(long double *)malloc(buffer_len*sizeof(long double));

	sprintf(mesh_file,"fxmesh%d.inp",number);

	inp_init(sim,&inp);
	inp_load_from_path(sim,&inp,get_input_path(sim),mesh_file);
	inp_check(sim,&inp,1.1);

	inp_reset_read(sim,&inp);

	while(1)
	{
		strcpy(temp,inp_get_string(sim,&inp));		//token
		if (strcmp(temp,"#ver")==0)
		{
			break;
		}
		sscanf(inp_get_string(sim,&inp),"%Le",&start);

		inp_get_string(sim,&inp);
		sscanf(inp_get_string(sim,&inp),"%Le",&stop);

		inp_get_string(sim,&inp);
		sscanf(inp_get_string(sim,&inp),"%Le",&points);

		inp_get_string(sim,&inp);
		sscanf(inp_get_string(sim,&inp),"%Le",&mul);

		dfx=(stop-start)/points;

		if ((dfx!=0.0)&&(mul!=0.0))
		{

			fx=start;

			while(fx<stop)
			{
				fx_mesh[ii]=fx;
				fx=fx+dfx;
				dfx=dfx*mul;
				ii++;
			}
		}
	}

	mesh_len=ii;
	mesh_pos=0;
	inp_free(sim,&inp);

}

void fx_step()
{
mesh_pos++;
}

int fx_points()
{
	return mesh_len;
}

int fx_run()
{
	if (mesh_pos<mesh_len)
	{
		return TRUE;
	}else
	{
		return FALSE;
	}
}

long double fx_get_fx()
{
	return fx_mesh[mesh_pos];
}

void fx_memory_free()
{
	free(fx_mesh);
}

