// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <util.h>
#include <dump_ctrl.h>
#include <complex_solver.h>
#include <const.h>
#include <light.h>

#include <device.h>
#include <light_interface.h>

#include <functions.h>
#include <log.h>
#include <lang.h>


//static gdouble min_light_error=1e-10;

EXPORT void light_dll_ver(struct simulation *sim)
{
        printf_log(sim,"%s\n",_("Full transfer matrix based light model"));
}


EXPORT int light_dll_solve_lam_slice(struct simulation *sim,struct light *in,int lam)
{
if (in->sun_E[lam]==0.0)
{
	return 0;
}
if (get_dump_status(sim,dump_optics)==TRUE)
{
	char one[100];
	sprintf(one,"%s %Lf nm\n",_("Solve optical slice at"),in->l[lam]*1e9);
	waveprint(sim,one,in->l[lam]*1e9);
}
int i;

gdouble complex Epl=0.0+0.0*I;
//gdouble complex Epc=0.0+0.0*I;
//gdouble complex Epr=0.0+0.0*I;

//gdouble complex Enl=0.0+0.0*I;
//gdouble complex Enc=0.0+0.0*I;
//gdouble complex Enr=0.0+0.0*I;

gdouble complex rc=0.0+0.0*I;
gdouble complex tc=0.0+0.0*I;
gdouble complex rr=0.0+0.0*I;
gdouble complex tr=0.0+0.0*I;

gdouble complex nbar_c=0.0;
gdouble complex xi_c=0.0;

gdouble complex nbar_r=0.0;
gdouble complex xi_r=0.0;

gdouble complex rl=0.0+0.0*I;
gdouble complex tl=0.0+0.0*I;
gdouble complex nbar_l=0.0;
gdouble complex xi_l=0.0;

gdouble complex fp=0.0;
gdouble complex fn=0.0;

int ittr=0;
int pos=0;
//gdouble error=1000;
gdouble l=in->l[lam];

//gdouble nc;

int quit=FALSE;
gdouble test=FALSE;
gdouble dx=in->x[2]-in->x[1];

do
{

	pos=0;

	for (i=0;i<in->points;i++)
	{

		rc=in->r[lam][i];
		tc=in->t[lam][i];



		if (i==0)
		{
			rl=in->r[lam][i];
			tl=in->t[lam][i];

			nbar_l=in->nbar[lam][i];
			Epl=in->sun_E[lam]+0.0*I;

		}else
		{
			rl=in->r[lam][i-1];
			tl=in->t[lam][i-1];

			nbar_l=in->nbar[lam][i-1];
			Epl=0.0;
		}

		if (i==in->points-1)
		{
			rr=in->r[lam][i];
			tr=in->t[lam][i];
			nbar_r=in->nbar[lam][i];
		}else
		{
			rr=in->r[lam][i+1];
			tr=in->t[lam][i+1];
			nbar_r=in->nbar[lam][i+1];
		}


		nbar_c=in->nbar[lam][i];

		xi_l=((2*PI)/l)*nbar_l;
		xi_c=((2*PI)/l)*nbar_c;
		xi_r=((2*PI)/l)*nbar_r;

		gdouble complex pa=-tl;
		gdouble complex pbp=cexp(xi_c*dx*I);

		gdouble complex pbn=rl*cexp(-xi_c*dx*I);

		gdouble complex na=-tc;//Enc
		gdouble complex nbp=rc*cexp(xi_r*dx*I);//Enr*
		gdouble complex nbn=cexp(-xi_r*dx*I);


		//getchar();
		fp=0.0+0.0*I;
		fn=0.0+0.0*I;

		if (i==0)
		{
			fp=Epl;
			pa=0.0+0.0*I;
		}else
		{
			fp=0.0+0.0*I;
		}




		if (i!=0)
		{
			in->Ti[pos]=i;
			in->Tj[pos]=i-1;
			in->Tx[pos]=(double)gcreal(pa);
			in->Txz[pos]=(double)gcimag(pa);
			pos++;
		}

		in->Ti[pos]=i;
		in->Tj[pos]=i;
		in->Tx[pos]=(double)gcreal(pbp);
		in->Txz[pos]=(double)gcimag(pbp);
		pos++;

		in->Ti[pos]=i;
		in->Tj[pos]=in->points+i;
		in->Tx[pos]=(double)gcreal(pbn);
		in->Txz[pos]=(double)gcimag(pbn);
		pos++;


		if (i!=in->points-1)
		{
			in->Ti[pos]=in->points+i;
			in->Tj[pos]=i+1;
			in->Tx[pos]=(double)gcreal(nbp);
			in->Txz[pos]=(double)gcimag(nbp);
			pos++;
		}

		in->Ti[pos]=in->points+i;
		in->Tj[pos]=in->points+i;
		in->Tx[pos]=(double)gcreal(na);
		in->Txz[pos]=(double)gcimag(na);
		pos++;

		if (i!=in->points-1)
		{
			in->Ti[pos]=in->points+i;
			in->Tj[pos]=in->points+i+1;
			in->Tx[pos]=(double)gcreal(nbn);
			in->Txz[pos]=(double)gcimag(nbn);
			pos++;
		}

		in->b[i]=(double)gcreal(fp);
		in->bz[i]=(double)gcimag(fp);

		in->b[in->points+i]=(double)gcreal(fn);
		in->bz[in->points+i]=(double)gcimag(fn);


	}

	//complex_solver_print(sim,in->M,in->N,in->Ti,in->Tj, in->Tx, in->Txz,in->b,in->bz);
	//complex_solver_dump_matrix(in->M,in->N,in->Ti,in->Tj, in->Tx, in->Txz,in->b,in->bz);

	if (pos!=in->N)
	{
		printf_log(sim,"%s %d %d\n",_("I can't solve the matrix because I don't have enough equations."),pos,in->N);
		exit(0);
	}

	complex_solver(sim,in->M,in->N,in->Ti,in->Tj, in->Tx, in->Txz,in->b,in->bz);

	for (i=0;i<in->points;i++)
	{
		gdouble update;

		update=in->b[i];
		in->Ep[lam][i]=(gdouble)update;

		update=in->bz[i];
		in->Epz[lam][i]=(gdouble)update;

		update=in->b[in->points+i];
		in->En[lam][i]=(gdouble)update;

		update=in->bz[in->points+i];
		in->Enz[lam][i]=(gdouble)update;
	}

	ittr++;

if (test==TRUE)
{
	if (ittr>1) quit=TRUE;
	getchar();
}else
{
	quit=TRUE;
}
}while(quit==FALSE);


//getchar();
return 0;
}


